/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * main.c - demo main module to test irmp decoder
 *
 * Copyright (c) 2009-2010 Frank Meyer - frank(at)fli4l.de
 *
 * $Id: main.c,v 1.5 2010/03/29 09:33:29 fm Exp $
 *
 * ATMEGA88 @ 8 MHz
 *
 * Fuses: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * uncomment this for codevision compiler:
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// #define CODEVISION                                                   // to use Codevision Compiler instead of gcc
 
#ifdef CODEVISION 
#include <mega8.h>
#include <stdio.h>
#define uint8_t     unsigned char
#define uint16_t    unsigned int
#define F_CPU       8000000            // change for Codevision here, if you use WinAVR, use Project -> Configuration Options instead

// register values from datasheet for ATMega88
#define OCIE1A      1
#define WGM12       3
#define CS10        0
#define UDRE0       5
#define TXEN0       3

#include "irmp.h"
#include "irmp.c"

#else // gcc compiler

#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "irmp.h"
#include "irmpconfig.h"


#endif  // CODEVISION


#ifndef F_CPU
#error F_CPU unkown
#endif

void
timer_init (void)
{
#ifdef CODEVISION
  OCR1AH  = ((F_CPU / F_INTERRUPTS) >> 8) & 0xFF;                           // compare value: 1/10000 of CPU frequency (upper byte)
  OCR1AL  = ((F_CPU / F_INTERRUPTS) - 1)  & 0xFF;                           // compare value: 1/10000 of CPU frequency (lower byte)
#else  // gcc
  OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                    // compare value: 1/10000 of CPU frequency
#endif  // CODEVISION
  TCCR1B  = (1 << WGM12) | (1 << CS10);                                     // switch CTC Mode on, set prescaler to 1

#if defined (__AVR_ATmega8__) || defined (__AVR_ATmega16__) || defined (__AVR_ATmega32__) || defined (__AVR_ATmega64__) || defined (__AVR_ATmega162__) 
  TIMSK  = 1 << OCIE1A;                                                     // OCIE1A: Interrupt by timer compare
#else
  TIMSK1  = 1 << OCIE1A;                                                    // OCIE1A: Interrupt by timer compare
#endif	// __AVR...
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// Timer 1 output compare A interrupt service routine
#ifdef CODEVISION
interrupt [TIM1_COMPA] void timer1_compa_isr(void)
#else  // CODEVISION
ISR(TIMER1_COMPA_vect)
#endif  // CODEVISION
{
  irmp_ISR();                                                           // call irmp ISR
  // call other timer interrupt routines...
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MAIN: main routine
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
#ifdef CODEVISION
// This is the main routine if you use Codevision C Compiler
void
main (void)
{
  IRMP_DATA irmp_data;

  #pragma optsize-
  // crystal oscillator division factor: 1
  CLKPR=0x80;
  CLKPR=0x00;
  #ifdef _OPTIMIZE_SIZE_
  #pragma optsize+
  #endif
  static uint8_t *Proto[]={"SIRCS","NEC","SAMSUNG","MATSUSH","KASEIKYO","RECS80","RC5(x)","DENON","RC6","SAMSG32","APPLE"};
  #define IRMP_APPLE_ADDRESS 0x77E1 


  #if IRMP_LOGGING == 0
  // USART initialization has to be done here if Logging is off
  // Communication Parameters: 8 Data, 1 Stop, No Parity
  // USART Receiver: Off
  // USART Transmitter: On
  // USART0 Mode: Asynchronous
  // USART Baud Rate: 9600
  #define BAUDRATE 9600L
  UCSR0A=0x00;
  UCSR0B=0x08;
  UCSR0C=0x06;
  UBRR0H = ((F_CPU+BAUDRATE*8)/(BAUDRATE*16)-1) >> 8;            // store baudrate (upper byte)
  UBRR0L = ((F_CPU+BAUDRATE*8)/(BAUDRATE*16)-1) & 0xFF;    
  #endif

  irmp_init();         // initialize rc5

  printf("IRMP V1.0\n");
  #if IRMP_LOGGING == 1
  printf("Logging Mode\n");
  #endif

  timer_init();                                                             // initialize timer
  #asm("sei");                                                                // enable interrupts

  for (;;)
  {
    if (irmp_get_data (&irmp_data))
    {
        // ir signal decoded, do something here...
        // irmp_data.protocol is the protocol, see irmp.h
        // irmp_data.address is the address/manufacturer code of ir sender
        // irmp_data.command is the command code
        #if IRMP_LOGGING != 1
        if((irmp_data.protocol == IRMP_NEC_PROTOCOL) && (irmp_data.address == IRMP_APPLE_ADDRESS))
          printf("Code: Apple\n");
        else printf("Code: %s\n",Proto[irmp_data.protocol-1]);
        printf("Address: 0x%.2X\n",irmp_data.address);
        printf("Command: 0x%.2X\n\n",irmp_data.command);
        #endif
    }
  }
}

#else  // gcc

// This is the main routine if you use GCC Compiler
int
main (void)
{
static char *Proto[]={"SIRCS","NEC","SAMSUMG","MATSUSH","KASEIKYO","RECS80","RC5(x)","DENON","RC6","SAMSUNG","APPLE","RECS80x","NUBERT", "BANG_OLUFSEN", "GRUNDIG"};
  IRMP_DATA irmp_data;
  char s[7];
  

#if IRMP_LOGGING == 0
#define UART_BAUD                               38400L
// calculate real baud rate:
#define UBRR_VAL                                ((F_CPU+UART_BAUD*8)/(UART_BAUD*16)-1)      // round
#define BAUD_REAL                               (F_CPU/(16*(UBRR_VAL+1)))                   // real baudrate
/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Initialize  UART
 *  @details  Initializes UART
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
void
uart_init (void)
{
    UCSRB |= (1<<TXEN);                                                         // activate UART0 TX
    UBRRH = UBRR_VAL >> 8;                                                       // store baudrate (upper byte)
    UBRRL = UBRR_VAL & 0xFF;                                                     // store baudrate (lower byte)
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Send character
 *  @details  Sends character
 *  @param    ch character to be transmitted
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
void
uart_putc (unsigned char ch)
{
    while (!(UCSRA & (1<<UDRE)))
    {
        ;
    }

    UDR = ch;
}
/*---------------------------------------------------------------------------------------------------------------------------------------------------
 *  Send string
 *  @details  Sends String
 *  @param    s string to be transmitted
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
void uart_puts(const char *s )
{
    while (*s) 
      uart_putc(*s++);
 
}/* uart_puts */



  uart_init();
  uart_putc(0x0a);
  uart_puts("IRMP-LCD V1.0 www.mikrocontroller-projekte.de");
  uart_putc(0x0a);
  #if IRMP_LOGGING == 1
  uart_puts("Logging Mode");
  uart_putc(0x0a);
  #endif

#endif  // IRMP_LOGGING==0

  irmp_init();                                                              // initialize rc5
  timer_init();                                                             // initialize timer
  sei ();                                                                   // enable interrupts

  for (;;)
  {
    if (irmp_get_data (&irmp_data))
    {
        // ir signal decoded, do something here...
        // irmp_data.protocol is the protocol, see irmp.h
        // irmp_data.address is the address/manufacturer code of ir sender
        // irmp_data.command is the command code
 
#if IRMP_LOGGING != 1


        uart_putc(0x0A);
        uart_puts("R: Code: ");
		uart_puts(Proto[irmp_data.protocol-1]);
        uart_puts(" A: ");
		uart_puts(itoa(irmp_data.address,s,16));
        uart_puts(" C: ");
		uart_puts(itoa(irmp_data.command,s,16));
        /* put light to Display if available */

if (irmp_data.flags & IRMP_FLAG_REPETITION)
    {
      uart_puts(" Dauer ");
    }
    else
    {
      uart_puts(" Einzeln ");
    }

#endif
    }

  }
}

#endif  // CODEVISION / gcc
